"use strict";
Object.defineProperty(exports, "__esModule", { value: true });
const tslib_1 = require("tslib");
const command_1 = require("@oclif/command");
const generate_module_1 = require("../generate/generate-module");
const execQuietly_1 = require("../generate/helpers/execQuietly");
const listr2_1 = require("listr2");
const path = tslib_1.__importStar(require("path"));
const fs = tslib_1.__importStar(require("fs"));
const findProjectMainPath_1 = require("../generate/helpers/findProjectMainPath");
const ListrHelper_1 = require("../generate/helpers/ListrHelper");
const runTypeGen = async (projectMainPath, appPrefix) => {
    const customCodegenConfig = path.join(projectMainPath, './codegen.js');
    let codegenConfigPath;
    if (fs.existsSync(customCodegenConfig)) {
        codegenConfigPath = customCodegenConfig;
    }
    else {
        codegenConfigPath = path.join(__dirname, '../generate/runtime-config-helpers/codegen.js');
    }
    await execQuietly_1.execQuietly(`APP_PREFIX=${appPrefix} npx graphql-codegen --config ${codegenConfigPath}`, {
        cwd: projectMainPath,
    });
};
const fixGenerated = async (projectMainPath) => {
    const customFixGenerated = path.join(projectMainPath, 'fix-generated.js');
    let fixGeneratedPath;
    if (fs.existsSync(customFixGenerated)) {
        fixGeneratedPath = customFixGenerated;
    }
    else {
        fixGeneratedPath = path.join(__dirname, '../generate/runtime-config-helpers/fix-generated.js');
    }
    await execQuietly_1.execQuietly(`node ${fixGeneratedPath}`, { cwd: projectMainPath });
};
const prettifyGenerated = async (projectMainPath, modulesPath = 'src') => {
    await execQuietly_1.execQuietly(`npx prettier --write "${modulesPath}/**/*.ts" "generated/**/*.ts" --loglevel error`, {
        cwd: projectMainPath,
    });
};
class Generate extends command_1.Command {
    async run() {
        const { flags } = this.parse(Generate);
        const projectMainPath = findProjectMainPath_1.findProjectMainPath();
        const tasks = new listr2_1.Listr([
            ListrHelper_1.newTask('Generating code', async () => generate_module_1.executeGeneration(flags.appPrefix, flags.generatedPrefix, flags.modulesPath)),
            ListrHelper_1.newTask('Generating types', async () => runTypeGen(projectMainPath, flags.appPrefix)),
            ListrHelper_1.newTask('Tweak the generated types', async () => fixGenerated(projectMainPath)),
            ListrHelper_1.newTask('Prettify the generated code', async () => prettifyGenerated(projectMainPath, flags.modulesPath)),
        ], { renderer: ListrHelper_1.ListrRenderer });
        // eslint-disable-next-line @typescript-eslint/no-unused-vars,@typescript-eslint/no-empty-function
        await tasks.run().catch(() => { });
    }
}
exports.default = Generate;
Generate.description = 'generate GraphQL code';
Generate.examples = ['$ chimp generate', '$ chimp generate -a ~src -g ~chimp-helpers'];
Generate.flags = {
    help: command_1.flags.help({ char: 'h' }),
    appPrefix: command_1.flags.string({
        char: 'a',
        description: 'prefix that points to the sourcecode of your app',
        default: '~app',
    }),
    generatedPrefix: command_1.flags.string({
        char: 'g',
        description: 'prefix that points to the generated by chimp helper code',
        default: '~generated',
    }),
    modulesPath: command_1.flags.string({
        char: 'p',
        description: 'path to the graphQL modules, only use if you are migrating an existing Apollo App and you want to use chimp only for a part of it',
    }),
};
