#!/usr/bin/env node
"use strict";
Object.defineProperty(exports, "__esModule", { value: true });
exports.executeGeneration = void 0;
const tslib_1 = require("tslib");
const get_module_infos_1 = tslib_1.__importDefault(require("./parse-graphql/get-module-infos"));
const fs = tslib_1.__importStar(require("fs"));
const path = tslib_1.__importStar(require("path"));
// @ts-ignore
const shelljs = tslib_1.__importStar(require("shelljs"));
const debug_1 = tslib_1.__importDefault(require("debug"));
const graphql_1 = require("graphql");
const pascal_case_1 = require("pascal-case");
const getModuleNames_1 = tslib_1.__importDefault(require("./parse-graphql/getModuleNames"));
const getFederatedEntities_1 = tslib_1.__importDefault(require("./parse-graphql/getFederatedEntities"));
const getInterfaces_1 = tslib_1.__importDefault(require("./parse-graphql/getInterfaces"));
const getScalars_1 = tslib_1.__importDefault(require("./parse-graphql/getScalars"));
// import checkIfGitStateClean from './helpers/checkIfGitStateClean';
const saveRenderedTemplate_1 = require("./helpers/saveRenderedTemplate");
const findProjectMainPath_1 = require("./helpers/findProjectMainPath");
const execQuietly_1 = require("./helpers/execQuietly");
const getUnions_1 = tslib_1.__importDefault(require("./parse-graphql/getUnions"));
const debug = debug_1.default('generate-module');
const generateSchema = async (projectMainPath) => {
    await execQuietly_1.execQuietly(`ts-node -r tsconfig-paths/register ./generated/graphql/printSchema.ts > ./schema.graphql`, {
        cwd: projectMainPath,
    });
};
exports.executeGeneration = async (appPrefix = '~app', generatedPrefix = '~generated', modulesPath = 'src/') => {
    const capitalize = (string) => string.charAt(0).toUpperCase() + string.slice(1);
    const projectMainPath = findProjectMainPath_1.findProjectMainPath();
    shelljs.mkdir('-p', `${projectMainPath}/generated/graphql/helpers`);
    // "Framework" "generated" files - initial generation
    const createCombineSchemas = () => {
        const templateName = './templates/combineSchemas.ts';
        const filePath = `${projectMainPath}/generated/graphql/`;
        const fileName = 'combineSchemas.ts';
        saveRenderedTemplate_1.saveRenderedTemplate(templateName, {}, filePath, fileName);
    };
    debug('createCombineSchemas');
    createCombineSchemas();
    const createPrintSchema = () => {
        const templateName = './templates/printSchema.ts';
        const filePath = `${projectMainPath}/generated/graphql/`;
        const fileName = 'printSchema.ts';
        saveRenderedTemplate_1.saveRenderedTemplate(templateName, {}, filePath, fileName);
    };
    debug('createPrintSchema');
    createPrintSchema();
    const createGenericDataModelSchema = () => {
        const templateName = './templates/genericDataModelSchema.graphql';
        const filePath = `${projectMainPath}/generated/graphql/`;
        const fileName = 'genericDataModelSchema.graphql';
        saveRenderedTemplate_1.saveRenderedTemplate(templateName, {}, filePath, fileName);
    };
    debug('createGenericDataModelSchema');
    createGenericDataModelSchema();
    const createFrameworkSchema = () => {
        const templateName = './templates/frameworkSchema.graphql';
        const filePath = `${projectMainPath}/generated/graphql/`;
        const fileName = 'frameworkSchema.graphql';
        saveRenderedTemplate_1.saveRenderedTemplate(templateName, {}, filePath, fileName);
    };
    debug('createFrameworkSchema');
    createFrameworkSchema();
    const createGetCodegenConfig = () => {
        const templateName = './templates/getCodegenConfig.js';
        const filePath = `${projectMainPath}/generated/graphql/`;
        const fileName = 'getCodegenConfig.js';
        saveRenderedTemplate_1.saveRenderedTemplate(templateName, { appPrefix }, filePath, fileName);
    };
    debug('createGetCodegenConfig');
    createGetCodegenConfig();
    const modulesResolvedPath = path.join(projectMainPath, modulesPath);
    const graphqlPaths = shelljs.ls(path.join(modulesResolvedPath, '**/*.graphql'));
    const moduleNames = getModuleNames_1.default(graphqlPaths, projectMainPath);
    const modules = get_module_infos_1.default(moduleNames);
    const createGlobalResolvers = () => {
        const templateName = './templates/resolvers.handlebars';
        const context = { modules };
        const filePath = `${projectMainPath}/generated/graphql/`;
        const fileName = 'resolvers.ts';
        saveRenderedTemplate_1.saveRenderedTemplate(templateName, context, filePath, fileName);
    };
    debug('createGlobalResolvers');
    createGlobalResolvers();
    // End of "Framework" "generated" files
    // Initial App Setup files
    debug('generateSchema');
    await generateSchema(projectMainPath);
    const globalSchemaString = fs.readFileSync(path.join(projectMainPath, 'schema.graphql')); // read file
    const createGlobalSchema = () => {
        const templateName = './templates/schema.ts';
        const context = { modules, schemaString: globalSchemaString.toString().replace(/`/g, '\\`'), generatedPrefix };
        const filePath = `${projectMainPath}/generated/graphql/`;
        const fileName = 'schema.ts';
        saveRenderedTemplate_1.saveRenderedTemplate(templateName, context, filePath, fileName);
    };
    debug('createGlobalSchema');
    createGlobalSchema();
    modules.forEach((module) => {
        const moduleName = module.name;
        const { graphqlFileRootPath } = module;
        const createQuery = (queryName, hasArguments) => {
            const templateName = './templates/query.handlebars';
            const context = {
                queryName,
                moduleName,
                hasArguments,
                generatedPrefix,
            };
            const filePath = `${projectMainPath}/src/${graphqlFileRootPath}/queries/`;
            const fileName = `${queryName}Query.ts`;
            const keepIfExists = true;
            saveRenderedTemplate_1.saveRenderedTemplate(templateName, context, filePath, fileName, keepIfExists);
        };
        const createQuerySpec = (queryName, hasArguments) => {
            const templateName = './templates/query.spec.handlebars';
            const context = {
                queryName,
                moduleName,
                hasArguments,
                generatedPrefix,
                pascalCasedArgName: `Query${pascal_case_1.pascalCase(queryName)}Args`,
            };
            const filePath = `${projectMainPath}/src/${graphqlFileRootPath}/queries/`;
            const fileName = `${queryName}Query.spec.ts`;
            const keepIfExists = true;
            saveRenderedTemplate_1.saveRenderedTemplate(templateName, context, filePath, fileName, keepIfExists);
        };
        const createQuerySpecWrapper = (queryName, hasArguments) => {
            const templateName = './templates/querySpecWrapper.handlebars';
            const context = {
                queryName,
                moduleName,
                hasArguments,
                generatedPrefix,
                appPrefix,
                graphqlFileRootPath,
                pascalCasedArgName: `Query${pascal_case_1.pascalCase(queryName)}Args`,
            };
            const filePath = `${projectMainPath}/generated/graphql/helpers/`;
            const fileName = `${queryName}QuerySpecWrapper.ts`;
            const keepIfExists = false;
            saveRenderedTemplate_1.saveRenderedTemplate(templateName, context, filePath, fileName, keepIfExists);
        };
        if (module.queries && module.queries.length > 0) {
            shelljs.mkdir('-p', `${projectMainPath}/src/${graphqlFileRootPath}/queries`);
            module.queries.forEach(({ name, hasArguments }) => {
                createQuery(name, hasArguments);
                createQuerySpec(name, hasArguments);
                createQuerySpecWrapper(name, hasArguments);
            });
        }
        const createMutation = (mutationName, hasArguments) => {
            const templateName = './templates/mutation.handlebars';
            const context = {
                mutationName,
                moduleName,
                hasArguments,
                generatedPrefix,
            };
            const filePath = `${projectMainPath}/src/${graphqlFileRootPath}/mutations/`;
            const fileName = `${mutationName}Mutation.ts`;
            const keepIfExists = true;
            saveRenderedTemplate_1.saveRenderedTemplate(templateName, context, filePath, fileName, keepIfExists);
        };
        const createMutationSpec = (mutationName, hasArguments) => {
            const templateName = './templates/mutation.spec.handlebars';
            const context = {
                mutationName,
                moduleName,
                hasArguments,
                generatedPrefix,
                appPrefix,
                graphqlFileRootPath,
                pascalCasedArgName: `Mutation${pascal_case_1.pascalCase(mutationName)}Args`,
            };
            const filePath = `${projectMainPath}/src/${graphqlFileRootPath}/mutations/`;
            const fileName = `${mutationName}Mutation.spec.ts`;
            const keepIfExists = true;
            saveRenderedTemplate_1.saveRenderedTemplate(templateName, context, filePath, fileName, keepIfExists);
        };
        const createMutationSpecWrapper = (mutationName, hasArguments) => {
            const templateName = './templates/mutationSpecWrapper.handlebars';
            const context = {
                mutationName,
                moduleName,
                hasArguments,
                generatedPrefix,
                appPrefix,
                graphqlFileRootPath,
                pascalCasedArgName: `Mutation${pascal_case_1.pascalCase(mutationName)}Args`,
            };
            const filePath = `${projectMainPath}/generated/graphql/helpers/`;
            const fileName = `${mutationName}MutationSpecWrapper.ts`;
            const keepIfExists = false;
            saveRenderedTemplate_1.saveRenderedTemplate(templateName, context, filePath, fileName, keepIfExists);
        };
        if (module.mutations && module.mutations.length > 0) {
            shelljs.mkdir('-p', `${projectMainPath}/src/${graphqlFileRootPath}/mutations`);
            module.mutations.forEach(({ name, hasArguments }) => {
                createMutation(name, hasArguments);
                createMutationSpec(name, hasArguments);
                createMutationSpecWrapper(name, hasArguments);
            });
        }
    });
    const createTypeResolvers = () => {
        modules.forEach(({ name, typeDefinitions, types, schemaString, queries, mutations, graphqlFileRootPath }) => {
            const typeResolvers = [];
            if (types) {
                debug(`create type resolvers for module ${name}`);
                const federatedEntities = getFederatedEntities_1.default(schemaString);
                const interfaces = getInterfaces_1.default(schemaString);
                const unions = getUnions_1.default(schemaString);
                // Leaving this for now
                // eslint-disable-next-line no-param-reassign
                schemaString = schemaString.replace(/extend type/g, 'type');
                const source = new graphql_1.Source(schemaString);
                const schema = graphql_1.buildSchema(source, { assumeValidSDL: true });
                shelljs.mkdir('-p', `${projectMainPath}/src/${graphqlFileRootPath}/types/`);
                const createResolveType = (resolverTypeName) => {
                    const templateName = './templates/typeTypeResolvers.handlebars';
                    const capitalizedFieldName = capitalize('__resolveType');
                    const context = {
                        typeName: resolverTypeName,
                        fieldName: '__resolveType',
                        moduleName: name,
                        resolveReferenceType: true,
                        capitalizedFieldName,
                        generatedPrefix,
                    };
                    const filePath = `${projectMainPath}/src/${graphqlFileRootPath}/types/`;
                    const fileName = `${resolverTypeName}${capitalizedFieldName}.ts`;
                    const keepIfExists = true;
                    saveRenderedTemplate_1.saveRenderedTemplate(templateName, context, filePath, fileName, keepIfExists);
                };
                const createResolveTypeSpec = (resoverTypeName) => {
                    const templateName = './templates/typeTypeResolvers.spec.handlebars';
                    const capitalizedFieldName = capitalize('__resolveType');
                    const context = {
                        typeName: resoverTypeName,
                        fieldName: '__resolveType',
                        moduleName: name,
                        hasArguments: false,
                        resolveReferenceType: true,
                        capitalizedFieldName,
                        generatedPrefix,
                    };
                    const filePath = `${projectMainPath}/src/${graphqlFileRootPath}/types/`;
                    const fileName = `${resoverTypeName}${capitalizedFieldName}.spec.ts`;
                    const keepIfExists = true;
                    saveRenderedTemplate_1.saveRenderedTemplate(templateName, context, filePath, fileName, keepIfExists);
                };
                const createResolveTypeSpecWrapper = (resolverTypeName) => {
                    const templateName = './templates/typeTypeResolversSpecWrapper.handlebars';
                    const capitalizedFieldName = capitalize('__resolveType');
                    const context = {
                        typeName: resolverTypeName,
                        fieldName: '__resolveType',
                        moduleName: name,
                        hasArguments: false,
                        resolveReferenceType: true,
                        capitalizedFieldName,
                        generatedPrefix,
                        appPrefix,
                        graphqlFileRootPath,
                    };
                    const filePath = `${projectMainPath}/generated/graphql/helpers/`;
                    const fileName = `${resolverTypeName}${capitalizedFieldName}SpecWrapper.ts`;
                    const keepIfExists = false;
                    saveRenderedTemplate_1.saveRenderedTemplate(templateName, context, filePath, fileName, keepIfExists);
                };
                interfaces.forEach((interfaceName) => {
                    createResolveType(interfaceName);
                    createResolveTypeSpec(interfaceName);
                    createResolveTypeSpecWrapper(interfaceName);
                    typeResolvers.push({
                        typeName: interfaceName,
                        fieldName: [{ name: '__resolveType', capitalizedName: capitalize('__resolveType') }],
                    });
                });
                unions.forEach((unionName) => {
                    createResolveType(unionName);
                    createResolveTypeSpec(unionName);
                    createResolveTypeSpecWrapper(unionName);
                    typeResolvers.push({
                        typeName: unionName,
                        fieldName: [{ name: '__resolveType', capitalizedName: capitalize('__resolveType') }],
                    });
                });
                typeDefinitions.forEach((typeDef) => {
                    var _a, _b;
                    let filtered = [];
                    let type = schema.getType(typeDef.name);
                    if (!type) {
                        const newSchemaString = schemaString.replace(`extend type ${typeDef.name}`, `type ${typeDef.name}`);
                        type = graphql_1.buildSchema(new graphql_1.Source(newSchemaString)).getType(typeDef.name);
                    }
                    if (type === null || type === void 0 ? void 0 : type.astNode) {
                        // @ts-ignore
                        filtered = type.astNode.fields.filter((field) => field.directives.find((d) => d.name.value === 'computed' ||
                            d.name.value === 'link' ||
                            d.name.value === 'requires' ||
                            d.name.value === 'map'));
                    }
                    let isFederatedAndExternal = false;
                    if (federatedEntities.find((e) => e === typeDef.name)) {
                        isFederatedAndExternal =
                            Boolean(type.astNode) &&
                                Boolean(
                                // @ts-ignore
                                type === null || 
                                // @ts-ignore
                                type === void 0 ? void 0 : 
                                // @ts-ignore
                                type.astNode.fields.find((field) => field.directives.find((d) => d.name.value === 'external')));
                        const foundComputed = (_b = (_a = type === null || type === void 0 ? void 0 : type.astNode) === null || _a === void 0 ? void 0 : _a.directives) === null || _b === void 0 ? void 0 : _b.find((d) => d.name.value === 'computed');
                        const notComputed = Boolean(!foundComputed);
                        // If it's a federated and external but NOT marked with a computed directive, we do not want to
                        // create the resolveReference files for it.
                        if (!(isFederatedAndExternal && notComputed)) {
                            filtered.push({ name: { value: '__resolveReference' }, resolveReferenceType: true });
                        }
                    }
                    filtered.forEach(({ name: { value }, resolveReferenceType }) => {
                        const templateName = './templates/typeTypeResolvers.handlebars';
                        const capitalizedFieldName = capitalize(value);
                        const context = {
                            typeName: typeDef.name,
                            fieldName: value,
                            moduleName: name,
                            resolveReferenceType,
                            capitalizedFieldName,
                            generatedPrefix,
                        };
                        const filePath = `${projectMainPath}/src/${graphqlFileRootPath}/types/`;
                        const fileName = `${typeDef.name}${capitalizedFieldName}.ts`;
                        const keepIfExists = true;
                        saveRenderedTemplate_1.saveRenderedTemplate(templateName, context, filePath, fileName, keepIfExists);
                    });
                    const createTypeFieldResolverSpec = (value, resolveReferenceType, resolverArguments) => {
                        const templateName = './templates/typeTypeResolvers.spec.handlebars';
                        const capitalizedFieldName = capitalize(value);
                        const context = {
                            typeName: typeDef.name,
                            fieldName: value,
                            moduleName: name,
                            hasArguments: resolverArguments && resolverArguments.length,
                            resolveReferenceType,
                            capitalizedFieldName,
                            generatedPrefix,
                            pascalCasedArgName: `${typeDef.name}${pascal_case_1.pascalCase(capitalizedFieldName)}Args`,
                        };
                        const filePath = `${projectMainPath}/src/${graphqlFileRootPath}/types/`;
                        const fileName = `${typeDef.name}${capitalizedFieldName}.spec.ts`;
                        const keepIfExists = true;
                        saveRenderedTemplate_1.saveRenderedTemplate(templateName, context, filePath, fileName, keepIfExists);
                    };
                    const createTypeFieldResolverSpecWrapper = (value, resolveReferenceType, resolverArguments) => {
                        const templateName = './templates/typeTypeResolversSpecWrapper.handlebars';
                        const capitalizedFieldName = capitalize(value);
                        const context = {
                            typeName: typeDef.name,
                            fieldName: value,
                            moduleName: name,
                            hasArguments: resolverArguments && resolverArguments.length,
                            resolveReferenceType,
                            capitalizedFieldName,
                            generatedPrefix,
                            appPrefix,
                            graphqlFileRootPath,
                            isFederatedAndExternal,
                            pascalCasedArgName: `${typeDef.name}${pascal_case_1.pascalCase(capitalizedFieldName)}Args`,
                        };
                        const filePath = `${projectMainPath}/generated/graphql/helpers/`;
                        const fileName = `${typeDef.name}${capitalizedFieldName}SpecWrapper.ts`;
                        const keepIfExists = true;
                        saveRenderedTemplate_1.saveRenderedTemplate(templateName, context, filePath, fileName, keepIfExists);
                    };
                    filtered.forEach(({ name: { value }, arguments: resolverArguments, resolveReferenceType }) => {
                        createTypeFieldResolverSpec(value, resolveReferenceType, resolverArguments);
                        createTypeFieldResolverSpecWrapper(value, resolveReferenceType, resolverArguments);
                    });
                    if (filtered.length > 0) {
                        typeResolvers.push({
                            typeName: typeDef.name,
                            fieldName: filtered.map(({ name: { value } }) => ({ name: value, capitalizedName: capitalize(value) })),
                        });
                    }
                });
            }
            const scalars = getScalars_1.default(schemaString);
            const createScalarResolvers = () => {
                if (scalars && scalars.length > 0) {
                    shelljs.mkdir('-p', `${projectMainPath}/src/${graphqlFileRootPath}/scalars/`);
                }
                scalars.forEach((scalarName) => {
                    const templateName = './templates/scalarResolver.handlebars';
                    const context = {
                        scalarName,
                        moduleName: name,
                        generatedPrefix,
                    };
                    const filePath = `${projectMainPath}/src/${graphqlFileRootPath}/scalars/`;
                    const fileName = `${scalarName}.ts`;
                    const keepIfExists = true;
                    saveRenderedTemplate_1.saveRenderedTemplate(templateName, context, filePath, fileName, keepIfExists);
                });
            };
            createScalarResolvers();
            const moduleName = name;
            const createModuleResolvers = () => {
                const templateName = './templates/moduleResolvers.handlebars';
                const context = {
                    moduleName,
                    queries,
                    mutations,
                    typeResolvers,
                    graphqlFileRootPath,
                    appPrefix,
                    scalars,
                };
                const filePath = `${projectMainPath}/generated/graphql/`;
                const fileName = `${moduleName}Resolvers.ts`;
                saveRenderedTemplate_1.saveRenderedTemplate(templateName, context, filePath, fileName);
            };
            createModuleResolvers();
        });
    };
    debug('createTypeResolvers');
    createTypeResolvers();
};
